
from AffichageOpenGL import *

import thread
import time


wxEVT_METTRE_A_JOUR_POSITION = wxNewEventType()
wxEVT_METTRE_A_JOUR_ZOOM = wxNewEventType()

def EVT_METTRE_A_JOUR_POSITION(win, func):
    win.Connect(-1, -1, wxEVT_METTRE_A_JOUR_POSITION, func)
def EVT_METTRE_A_JOUR_ZOOM(win, func):
    win.Connect(-1, -1, wxEVT_METTRE_A_JOUR_ZOOM, func)
    
class MettreAJourPosition(wxPyEvent):
    def __init__(self, position):
        wxPyEvent.__init__(self)
        self.SetEventType(wxEVT_METTRE_A_JOUR_POSITION)
        self.position = position
        
class MettreAJourZoom(wxPyEvent):
    def __init__(self, deltaY):
        wxPyEvent.__init__(self)
        self.SetEventType(wxEVT_METTRE_A_JOUR_ZOOM)
        self.deltaY = deltaY

   
class ThreadTortue:
    def __init__(self, vueObj):
        self.vueObj = vueObj
        self.delta = (0.0, 0.0)
        self.vitesse = 0.01
        
    def setDelta(self, delta):
        self.delta = delta
        
    def start(self):
        self.delta = (0.0, 0.0)
        self.keepGoing = self.running = true
        thread.start_new_thread(self.run, ())

    def stop(self):
        self.keepGoing = false

    def run(self):
        while self.keepGoing:
            evt = MettreAJourPosition(self.delta)
            wxPostEvent(self.vueObj, evt)
            time.sleep(0.02)
            
        self.running = false 
        
class ThreadZoom:
    def __init__(self, vueObj):
        self.vueObj = vueObj
        self.posY = 0.0
        self.vitesse = 0.01
        
    def setDeltaY(self, deltaY):
        self.deltaY = deltaY
        
    def start(self):
        self.deltaY = 0.0
        self.keepGoing = self.running = true
        thread.start_new_thread(self.run, ())

    def stop(self):
        self.keepGoing = false

    def run(self):
        while self.keepGoing:
            evt = MettreAJourZoom(self.deltaY)
            wxPostEvent(self.vueObj, evt)
            time.sleep(0.02)
            
        self.running = false 
        
class Vue(AffichageOpenGL):
    def __init__(self, attache):
        AffichageOpenGL.__init__(self, attache)
        
        self.zoomMax = 200.0
        self.zoomMin = 0.1       
        
        self.positionMax = (5000.0, 5000.0)
        self.positionMin = (-5000.0, -5000.0)
        
        self.zoom = 6.0
        self.position = (0.0, 0.0)
        self.threadTortue = ThreadTortue(self)        
        self.threadZoom = ThreadZoom(self)   

        
        EVT_METTRE_A_JOUR_POSITION(self, self.evChangePosition)
        EVT_METTRE_A_JOUR_ZOOM(self, self.evChangeZoom)
        
        EVT_SIZE(self, self.OnSize)
        EVT_LEFT_DOWN(self, self.OnMouseDown)  # needs fixing...
        EVT_LEFT_UP(self, self.OnMouseUp)
        EVT_RIGHT_DOWN(self, self.OnMouseDown)  # needs fixing...
        EVT_RIGHT_UP(self, self.OnMouseUp)
        EVT_MOTION(self, self.OnMouseMotion)
        
    def setPosition(self, position) : 
        if position[0] <= self.positionMax[0] and position[0] >= self.positionMin[0]:
            self.position = (position[0], self.position[1])
        else :
            if position[0] > self.positionMax[0] :
                self.position = (self.positionMax[0], self.position[1])
            elif position[0] < self.positionMin[0] :
                self.position = (self.positionMin[0], self.position[1])
         
        if position[1] <= self.positionMax[1] and position[1] >= self.positionMin[1]: 
            self.position = (self.position[0], position[1])
        else :
            if position[1] > self.positionMax[1] :
                self.position = (self.position[0], self.positionMax[1])
            elif position[1] < self.positionMin[1] :
                self.position = (self.position[0], self.positionMin[1])
    def setZoom(self, zoom) :
        if zoom <= self.zoomMax and zoom >= self.zoomMin :
            self.zoom = zoom
        else :
            if zoom > self.zoomMax :
                self.zoom = self.zoomMax
            elif zoom < self.zoomMin :
                self.zoom = self.zoomMin
                
    def evChangePosition(self, evt):
        self.setPosition ((self.position[0]+0.1*evt.position[0]/self.zoom,
                          self.position[1]+0.1*evt.position[1]/self.zoom))
        self.repositionner()      
        self.Refresh(false)    
        
    def evChangeZoom(self, evt):
        self.setZoom(self.zoom + evt.deltaY * log(self.zoom+1.0)*0.01)
        self.repositionner()      
        self.Refresh(false)  
        
    def repositionner(self): 
              
           
        size = self.GetClientSize()
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        if size.width > 0.0 and size.height > 0.0 :
            glOrtho(0, size.width/self.zoom, 0, size.height/self.zoom, 100.0, -100.0)
        glViewport(0,0, size.width, size.height)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glTranslatef(size.width/(self.zoom*2)-self.position[0],
                     size.height/(self.zoom*2)-self.position[1], 0.0); 
                     
        
    def OnSize(self, event):
        if self.GetContext():
            self.SetCurrent()      
            self.repositionner()

    def OnMouseDown(self, evt):
        self.CaptureMouse()
        self.xDebut, self.yDebut = evt.GetPosition()
        if evt.LeftIsDown() :
            self.threadTortue.start()
        elif evt.RightIsDown() :
            self.threadZoom.start()

    def OnMouseUp(self, evt):
        if self.HasCapture():
            self.ReleaseMouse()
            if evt.LeftUp() :
                self.threadTortue.stop()
            elif evt.RightUp() :
                self.threadZoom.stop()

    def OnMouseMotion(self, evt):
        if evt.Dragging():
                            
            self.x, self.y = evt.GetPosition()
            if evt.LeftIsDown() :
                self.threadTortue.setDelta(((self.x-self.xDebut), (self.yDebut-self.y)))
            elif evt.RightIsDown() :
                self.threadZoom.setDeltaY(self.yDebut-self.y)

